"""
#######################################################################
#
# Licensed Materials - Property of Pipelinefx L.L.C.
#
#
# (C) COPYRIGHT Pipelinefx Limited Liability Corporation.
#  All Rights Reserved.
#                               
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with 
#  Pipelinefx L.L.C.
#
# $DateTime: 2012/10/12 16:20:55 $
# $Revision: #2 $
# $Change: 10156 $
# $Id: //depot/main/qubeMobileServer/qubeMobile/model/qbUILogList.py#2 $
# $File: //depot/main/qubeMobileServer/qubeMobile/model/qbUILogList.py $
#
# Written by: Brian Knudson / PFX
#             briank@pipelinefx.com
#             2012-02-17
#
# Description:
#  UILogList class - Holds a list of a qbUILogs - containers for
#  job logs with sensible methods.
#
#
#######################################################################
"""

import os
import sys
import re
import time
import datetime
import logging

try:
    import qb
except ImportError:
    for api_path in ("/Applications/pfx/qube/api/python/",
                     "/usr/local/pfx/qube/api/python/",
                     "C:\Program Files\pfx\qube\api\python"):
        if api_path not in sys.path and os.path.exists(api_path):
            sys.path.insert(0,api_path)
    import qb


from qbUIList import QbUIList
from qbUILog import QbUILog


"""
Qb UI LogList Class
"""
    


class QbUILogList(QbUIList):
    """
    A log list model.
    Can be contructed:
      blank: which will get all hosts from the db
      list: will create an internal dict from the list of hosts
      dict: will copy the given dict to internal
    """
    def __init__(self, job_list, query_SQL):
        logging.debug("Creating QbUILogList")
        super(QbUILogList,self).__init__()

        self.__job_list = job_list
        self.query_SQL = query_SQL

        # one stat for all logs rather than one stat per log if it was done in the
        # log...  being in the log seems to make more OO sense, but.. too expensive)
        try:
            self.__mounted_logs = os.path.exists(qb.localconfig().get("client_logpath"))
            if self.__mounted_logs:
                logging.info("Use logfile location specified by client_ligpath: '%s'" % qb.localconfig().get("client_logpath"))
            else:
                logging.info("Fetching logfiles from supervisor")
        except:
            logging.info("Fetching logfiles from supervisor")
            self.__mounted_logs = False

    def __del__(self):
        """
        Destructor - informs everyone to clear their cache files
        """
        logging.debug("Log list destructor")
        self.clearAll()


    def __getitem__(self,key):
        """
        Implementation of self[key] on the fetching side.
        self[key] will be a qb.Job.
        """
        if not self._data.has_key(key):
            self._data[key] = QbUILog(job_id=key,job_list=self.__job_list,
                                      mounted_logs=self.__mounted_logs,query_SQL=selfquery_SQL)

        return self._data[key]


    def clearAll(self):
        """
        Goes to each log in the list & tells 'em to clean up.
        """
        logging.debug("Log list clearing cache files...")
        for lid,log in self._data.items():
            log.clearCache()
        self._data.clear() # maybe redundant?
        self._data = {}
        

