"""
#######################################################################
#
# Licensed Materials - Property of Pipelinefx L.L.C.
#
#
# (C) COPYRIGHT Pipelinefx Limited Liability Corporation.
#  All Rights Reserved.
#                               
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with 
#  Pipelinefx L.L.C.
#
# $DateTime: 2012/10/12 16:20:55 $
# $Revision: #4 $
# $Change: 10156 $
# $Id: //depot/main/qubeMobileServer/util/makeCert.py#4 $
# $File: //depot/main/qubeMobileServer/util/makeCert.py $
#
# Written by: Brian Knudson / PFX
#             briank@pipelinefx.com
#             2012-06-04
#
# Description:
#  Python-based web server, designed specifically for qubeMobileView
#
# Notes:
#  - Simply run, no args needed: python webserver.py
#  - Nothing outside of a typical python install is required for normal
#    use, however, qubeMobileView relies on mako templates, so you'll
#    likely need to have python-mako installed.  This should be available
#    via your favorite package manager, or via pip: 'pip install Mako'
#  - Likewise, the models will make use of python MySQLdb if it is
#    available.  Using MySQLdb will allow for less communication
#    and less stress on the supervisor.  It is highly recommended
#    that you instal python MySQLdb.
#
#######################################################################
"""

import time
import logging
from socket import gethostname
from time import gmtime, mktime
from os.path import exists, join

CERT_FILENAME = "qubeMobileServer.crt"
KEY_FILENAME = "qubeMobileServer.key"

def createSelfSignedCert(cert_dir):
    """
    If datacard.crt and datacard.key don't exist in cert_dir, create a new
    self-signed cert and keypair and write them into that directory.
    """
    cert_file = join(cert_dir, CERT_FILENAME)
    key_file = join(cert_dir, KEY_FILENAME)

    if exists(cert_file) and exists(key_file):
	# TODO: Validate the files if they already exist
	return cert_file,key_file

    import sys
    cert_command = "openssl req -new -x509 -days 365 -nodes -out cert.pem -keyout key.pem"
    if sys.platform == 'win32':
        logging.error("Cannot create self-signed certificates on Windows.")
        logging.error("Please supply a certificate or run with ssl, using the --no-ssl flag on launch.")
        logging.error("You can generate a certificate and key file with the following command:\n\n  %s\n\n" % cert_command)
        logging.error("You will then need to specify the location of these files either in the conf file or at launch.")
        sys.exit(1)

    try:
        from OpenSSL import crypto
    except ImportError,e:
        logging.error("\nCould not import OpenSSL module needed for creating certificates:")
        logging.error(e)
        logging.error("You can either install pyOpenSSL (https://launchpad.net/pyopenssl),")
        logging.error("run in non-ssl mode (Use --no-ssl options to run without ssl),")
        logging.error("or create/use your own certficates.")
        logging.error("The following command will generate a certificate:\n\n%s\n\n" % cert_command)
        sys.exit(1)

    cert_file = join(cert_dir, CERT_FILENAME)
    key_file = join(cert_dir, KEY_FILENAME)
        
    if not exists(cert_file) or not exists(key_file):
            
        # create a key pair
        k = crypto.PKey()
        k.generate_key(crypto.TYPE_RSA, 1024)

        # create a self-signed cert
        cert = crypto.X509()
        cert.get_subject().C = "US"
        cert.get_subject().ST = "Honolulu"
        cert.get_subject().L = "Haiwaii"
        cert.get_subject().O = "PipelineFX"
        cert.get_subject().OU = "PipelinFX"
        cert.get_subject().CN = gethostname()
        cert.set_serial_number(1000)
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(10*365*24*60*60)
        cert.set_issuer(cert.get_subject())
        cert.set_pubkey(k)
        cert.sign(k, 'sha1')

        logging.info("Creating certficate file: %s" % cert_file)
        open(cert_file, "wt").write(
            crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
        logging.info("Creating key file: %s" % key_file)
        open(key_file, "wt").write(
            crypto.dump_privatekey(crypto.FILETYPE_PEM, k))

    # give the cert time flush to disk
    time.sleep(0.5)
    return cert_file,key_file
